/***************************************************************************
                            kde_KAccel.h -  description
                             -------------------
    begin                : Sun Nov 24 16:54:36 2002
    copyright            : (C) 2000-2001 Lost Highway Ltd. All rights reserved.
    email                : Lost_Highway@tipitina.demon.co.uk
    generated by         : duke@tipitina on Sun Nov 24 16:54:36 2002, using kalyptus 0.9.
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#ifndef KDE_KACCEL
#define KDE_KACCEL

#include <qtc/qtc_types.h>

#include <kdec/kde_types.h>

/**
*
 Handle shortcuts.

 Allow a user to configure shortcuts
 through application configuration files or through the
 @ref KKeyChooser GUI. 

 A KAccel contains a list of accelerator actions.

 For example, CTRL+Key_P could be a shortcut for printing a document. The key
 codes are listed in qnamespace.h. "Print" could be the action name for printing.
 The action name identifies the shortcut in configuration files and the
 @ref KKeyChooser GUI.

 A KAccel object handles key events sent to its parent widget and to all
 children of this parent widget.  The most recently created KAccel object
 has precedence over any KAccel objects created before it.
 When a shortcut pressed, KAccel calls the slot to which it has been
 connected. If you want to set global accelerators, independent of the window
 which has the focus, use @ref KGlobalAccel.

 Reconfiguration of a given shortcut can be prevented by specifying
 that an accelerator item is not configurable when it is inserted. A special
 group of non-configurable key bindings are known as the
 standard accelerators.

 The standard accelerators appear repeatedly in applications for
 standard document actions such as printing and saving. A convenience method is
 available to insert and connect these accelerators which are configurable on
 a desktop-wide basis.

 It is possible for a user to choose to have no key associated with
 an action.

 The translated first argument for @ref insertItem() is used only
 in the configuration dialog.
<pre>
 KAccel* pAccel = new KAccel( this );

 pAccel->insert( "Scroll Up", i18n("Scroll up"),
                       i18n("Scroll up the current document by one line."),
                       Qt::Key_Up, this, SLOT(slotScrollUp()) );
 pAccel->insert( KStdAccel::Print, this, SLOT(slotPrint()) );

 pAccel->readSettings();
</pre>

 @version $Id: kde_KAccel.h,v 1.5 2002/12/30 14:37:05 geiseri Exp $
 
 @short Configurable shortcut support for widgets.

*/

/** *
 Creates a new KAccel that watches @p pParent, which is also
 the QObject's parent. 

*/
#ifdef __cplusplus
extern   "C" {
#endif
kde_KAccel * kde_new_KAccel(qt_QWidget * pParent, char* psName);

/** *
 Creates a new KAccel that watches @p watch.

*/
kde_KAccel * kde_new_KAccel1(qt_QWidget * watch, qt_QObject * parent, char* psName);
void kde_del_KAccel( kde_KAccel* p );

/** *

 Returns the KAccel's @p KAccelActions, a list of @p KAccepAction.

*/

/** *

 Returns the KAccel's @p KAccelActions, a list of @p KAccepAction.

*/

/** *
 Checks whether the KAccel is active.

*/
int kde_KAccel_isEnabled(kde_KAccel* instPointer);

/** *
 Enables or disables the KAccel.

*/
void kde_KAccel_setEnabled(kde_KAccel* instPointer,int bEnabled);

/** *
 Checks whether auto-update of connections is enabled.

*/
int kde_KAccel_getAutoUpdate(kde_KAccel* instPointer);

/** *
 Enable auto-update of connections. This means that the signals
 are automatically disconnected when you disable an action, and
 re-enabled when you enable it. By default auto update is turned 
 on. If you disable auto-update, you need to call 
 @ref updateConnections() after changing actions.

*/
int kde_KAccel_setAutoUpdate(kde_KAccel* instPointer,int bAuto);

/** *
 Create an accelerator action.

 Usage:
<pre>
 insert( "Do Something", i18n("Do Something"),
   i18n("This action allows you to do something really great with this program to "
        "the currently open document."),
   ALT+Key_D, this, SLOT(slotDoSomething()) );
</pre>

*/

/** *
 Same as first @ref insert(), but with separate shortcuts defined for
 3- and 4- modifier defaults.
	 
*/

/** *
 This is an overloaded function provided for convenience.
 The advantage of this is when you want to use the same text for the name
 of the action as for the user-visible label.

 Usage: 
 <pre>
 insert( I18N_NOOP("Do Something"), ALT+Key_D, this, SLOT(slotDoSomething()) );
 </pre>

*/

/** *
 Similar to the first @ref insert() method, but with the action
 name, short description, help text, and default shortcuts all
 set according to one of the standard accelerators.

*/

/** *
 Use this to insert a label into the action list.  This will be
 displayed when the user configures shortcuts.

*/

/** *
 Removes the accelerator action identified by the name.
 Remember to also call updateConnections().

*/
int kde_KAccel_remove(kde_KAccel* instPointer,qt_QString * sAction);

/** *
 Updates the connections of the accelerations after changing them. 
 This is only neccessary if you have disabled auto updates which are
 on by default.

*/
int kde_KAccel_updateConnections(kde_KAccel* instPointer);

/** *
 Return the shortcut associated with the action named by @p sAction.

*/
kde_KShortcut* kde_KAccel_shortcut(kde_KAccel* instPointer,qt_QString * sAction);

/** *
 Set the shortcut to be associated with the action named by @p sAction.

*/
int kde_KAccel_setShortcut(kde_KAccel* instPointer,qt_QString * sAction, kde_KShortcut* shortcut);

/** *
 Set the slot to be called when the shortcut of the action named
 by @p sAction is pressed.

*/
int kde_KAccel_setSlot(kde_KAccel* instPointer,qt_QString * sAction, qt_QObject * pObjSlot, char* psMethodSlot);

/** *
 Enable or disable the action named by @p sAction.

*/
int kde_KAccel_setEnabled1(kde_KAccel* instPointer,qt_QString * sAction, int bEnabled);

/** *
 Returns the configuration group of the settings.

*/
qt_QString * kde_KAccel_configGroup(kde_KAccel* instPointer);

/** *
 Returns the configuration group of the settings.

*/
void kde_KAccel_setConfigGroup(kde_KAccel* instPointer,qt_QString * name);

/** *
 Read all shortcuts from @p pConfig, or (if @p pConfig
 is zero) from the application's configuration file
 @ref KGlobal::config().

 The group in which the configuration is stored can be
 set with @ref setConfigGroup().

*/
int kde_KAccel_readSettings(kde_KAccel* instPointer,kde_KConfigBase* pConfig);

/** *
 Write the current shortcuts to @p pConfig,
 or (if @p pConfig is zero) to the application's
 configuration file.

*/
int kde_KAccel_writeSettings(kde_KAccel* instPointer,kde_KConfigBase* pConfig);

/** *
 Emits the @ref keycodeChanged() signal.
	 
*/
void kde_KAccel_emitKeycodeChanged(kde_KAccel* instPointer);

/** * 

	 
*/
int kde_KAccel_insertItem(kde_KAccel* instPointer,qt_QString * sLabel, qt_QString * sAction, char* psKey, int nIDMenu, qt_QPopupMenu* pMenu, int bConfigurable);

/** * 

	 
*/
int kde_KAccel_insertItem1(kde_KAccel* instPointer,qt_QString * sLabel, qt_QString * sAction, int key, int nIDMenu, qt_QPopupMenu* pMenu, int bConfigurable);

/** * 

	 
*/
int kde_KAccel_insertStdItem(kde_KAccel* instPointer,int identifier, qt_QString * descr);

/** * 

	 
*/
int kde_KAccel_connectItem(kde_KAccel* instPointer,qt_QString * sAction, qt_QObject * pObjSlot, char* psMethodSlot, int bActivate);

/** * 

	 
*/
int kde_KAccel_connectItem1(kde_KAccel* instPointer,int accel, qt_QObject * pObjSlot, char* psMethodSlot);

/** * 

	 
*/
int kde_KAccel_removeItem(kde_KAccel* instPointer,qt_QString * sAction);

/** * 

	 
*/
int kde_KAccel_setItemEnabled(kde_KAccel* instPointer,qt_QString * sAction, int bEnable);

/** * 

	 
*/
void kde_KAccel_changeMenuAccel(kde_KAccel* instPointer,qt_QPopupMenu* menu, int identifier, qt_QString * action);

/** * 

	 
*/
void kde_KAccel_changeMenuAccel1(kde_KAccel* instPointer,qt_QPopupMenu* menu, int identifier, int accel);

/** *

 Retrieve the key code of the accelerator item with the action name
 @p action, or zero if either the action name cannot be
 found or the current key is set to no key.
	 
*/
int kde_KAccel_currentKey(kde_KAccel* instPointer,qt_QString * action);

/** *

 Return the name of the accelerator item with the keycode @p key,
 or QString::null if the item cannot be found.
	 
*/
qt_QString * kde_KAccel_findKey(kde_KAccel* instPointer,int key);

/** * 

	 
*/
int kde_KAccel_stringToKey( qt_QString * arg1);
void kde_KAccel_virtual_hook(kde_KAccel* instPointer,int identifier, void* data);

#ifdef __cplusplus
}
#endif
#endif
