/***************************************************************************
                            StyleSheet.h -  description
                             -------------------
    begin                : Fri Mar  8 13:55:08 2002
    copyright            : (C) 2000-2002 Lost Highway Ltd. All rights reserved.
    email                : Lost_Highway@tipitina.demon.co.uk
    generated by         : duke@tipitina on Fri Mar  8 13:55:08 2002, using kalyptus 0.9.
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#ifndef STYLESHEET_H
#define STYLESHEET_H

#include <Foundation/NSObject.h>
#include <Foundation/NSString.h>
#include <qtobjc/QtSupport.h>

/**

 The <code> StyleSheet </code> interface is the abstract base
 interface for any type of style sheet. It represents a single style
 sheet associated with a structured document. In HTML, the
 StyleSheet interface represents either an external style sheet,
 included via the HTML <a
 href="http://www.w3.org/TR/REC-html40/struct/links.-[html h]-12.3">
 LINK </a> element, or an inline <a
 href="http://www.w3.org/TR/REC-html40/present/styles.-[html h]-14.2.3">
 STYLE </a> element. In XML, this interface represents an external
 style sheet, included via a <a
 href="http://www.w3.org/TR/xml-stylesheet"> style sheet processing
 instruction </a> .

 
 @short 
 The <code> StyleSheet </code> interface is the abstract base
 interface for any type of style sheet.
*/
@class DOMNode; 
@class MediaList; 

@interface  StyleSheet : NSObject <QtSupport>
{
@private
	void * _qt;
	BOOL  _allocatedInObjcWorld;
}
- init;
- copyWithZone: (NSZone *) zone;
- (void) dealloc;

/** 
 This specifies the style sheet language for this style sheet.
 The style sheet language is specified as a content type (e.g.
 "text/css"). The content type is often specified in the <code>
 ownerNode </code> . A list of registered content types can be
 found at <a
 href="ftp://ftp.isi.edu/in-notes/iana/assignments/media-types/">
 ftp://ftp.isi.edu/in-notes/iana/assignments/media-types/ </a> .
 Also see the <a
 href="http://www.w3.org/TR/REC-html40/struct/links.-[html adef]-type-A">
 type attribute definition </a> for the <code> LINK </code>
 element in HTML 4.0, and the type pseudo-attribute for the XML
 <a href="http://www.w3.org/TR/xml-stylesheet"> style sheet
 processing instruction </a> .

     
*/
- (NSString *) type;

/** 
 <code> false </code> if the style sheet is applied to the
 document. <code> true </code> if it is not. Modifying this
 attribute may cause a reresolution of style for the document.

     
*/
- (BOOL) disabled;

/** 
 see @ref disabled
     
*/
- setDisabled: (BOOL)arg1;

/** 
 The node that associates this style sheet with the document.
 For HTML, this may be the corresponding <code> LINK </code> or
 <code> STYLE </code> element. For XML, it may be the linking
 processing instruction. For style sheets that are included by
 other style sheets, this attribute has a value of null.

     
*/
//- (DOMNode*) ownerNode;

/** 
 For style sheet languages that support the concept of style
 sheet inclusion, this attribute represents the including style
 sheet, if one exists. If the style sheet is a top-level style
 sheet, or the style sheet language does not support inclusion,
 the value of the attribute is null.

     
*/
- (StyleSheet*) parentStyleSheet;

/** 
 If the style sheet is a linked style sheet, the value of its
 attribute is its location. For inline style sheets, the value
 of this attribute is null. See the <a
 href="http://www.w3.org/TR/REC-html40/struct/links.-[html adef]-href">
 href attribute definition </a> for the <code> LINK </code>
 element in HTML 4.0, and the href pseudo-attribute for the XML
 <a href="http://www.w3.org/TR/xml-stylesheet"> style sheet
 processing instruction </a> .

     
*/
//- (NSString *) href;

/** 
 The advisory title. The title is often specified in the <code>
 ownerNode </code> . See the <a
 href="http://www.w3.org/TR/REC-html40/struct/global.-[html adef]-title">
 title attribute definition </a> for the <code> LINK </code>
 element in HTML 4.0, and the title pseudo-attribute for the XML
 <a href="http://www.w3.org/TR/xml-stylesheet"> style sheet
 processing instruction </a> .

     
*/
//- (NSString *) title;

/** 
 The intended destination media for style information. The media
 is often specified in the <code> ownerNode </code> . See the <a
 href="http://www.w3.org/TR/REC-html40/present/styles.-[html adef]-media">
 media attribute definition </a> for the <code> LINK </code>
 element in HTML 4.0, and the media pseudo-attribute for the XML
 <a href="http://www.w3.org/TR/WD-xml-stylesheet"> style sheet
 processing instruction </a> .

     
*/
//- (MediaList) media;

/** 

     
*/
- (BOOL) isCSSStyleSheet;
- (BOOL) isNull;
@end

#endif
